//
//  AppController.mm
//  tutorial
//
//  Created by Tobias Stamm on 30.05.05.
//  Copyright 2005 __MyCompanyName__. All rights reserved.
//

#import "AppController.h"

@implementation AppController


///////////////////
//Initialisation-Function
  - (IBAction)setup:(id)sender
  {
		//Kill all previous allocations and stop the timer
		destruct();
		
		//Create a new root
		root = new SoSeparator();
		root->ref();
		
		//Do all the initialisation stuff of the scene
		init();

		//Create a new timer
		timer = new SoTimerSensor(idle,nil);
		timer->setInterval(SbTime(FPSInterval));
		
		//Display the scene
		[scenegraph setRoot:root];
		[scenegraph viewAll];
		
  }


/////////////////
//Start/Pause the timer without resetting
  - (IBAction)startstop:(id)sender
  {
		if(timer){
			if(running){
				timer->unschedule();
				running=0;
			}else{
				timer->schedule();
				running=1;
			}
		}
  }


/////////////////
//Quit
  - (IBAction)quit:(id)sender
  {
		//Kill everything
		destruct();
		[[NSApplication sharedApplication]terminate:self];
  }


//////////////////////
//The keyboard-event-function
- (IBAction)key:(id)sender{
	char c;
	[[sender keyEquivalent] getCString:&c];
	keyevent(c);
}


@end

/////////////////////
//Delete all Elements and the timer
void destruct(){
	if(root){root->unref();root=NULL;}
	if(timer){delete timer;timer=NULL;}
	t=0;
	running=0;
}


/////////////////////
//c++-Init-Function. Setting up the scene
void init(){
	for(int x=0;x<MAX_X;x++){
		for(int y=0;y<MAX_Y;y++){
			cubes[x][y][0].h=INITIAL_HEIGHT;
			cubes[x][y][0].b=0;
			cubes[x][y][0].c=1;
			cubes[x][y][MAX_Z].c=0;
		}
	}
	for(int x=0;x<MAX_X;x++){
		for(int y=0;y<MAX_Y;y++){
			for(int z=1;z<=MAX_Z;z++){
				cubes[x][y][z].sep = new SoSeparator();
				cubes[x][y][z].tran = new SoTranslation();
				cubes[x][y][z].cube = new SoCube();
				cubes[x][y][z].cube->width.setValue(1-BOXES_SEPARATION);
				cubes[x][y][z].cube->height.setValue(1-BOXES_SEPARATION);
				cubes[x][y][z].h=INITIAL_HEIGHT;
				cubes[x][y][z].b=z*INITIAL_HEIGHT;
				cubes[x][y][z].c=1;
				cubes[x][y][z].sep->addChild(cubes[x][y][z].tran);
				cubes[x][y][z].sep->addChild(cubes[x][y][z].cube);
				root->addChild(cubes[x][y][z].sep);
			}
		}
	}
	for(int x=0;x<MAX_X;x++){
		for(int y=0;y<MAX_Y;y++){
			for(int z=0;z<=MAX_Z;z++){
				heightbuffer[x][y][z]=INITIAL_HEIGHT;
				heightbuffer2[x][y][z]=INITIAL_HEIGHT;
			}
		}
	}
	cubes[MAX_X/2][MAX_Y/2][1].h=2;
	updatescene();
}

//////////////////////
//The keyboard-event-function
void keyevent(char key){
	//printf("Keyboard: %c",key);
}

//////////////////
//The idle-function. Calculating the t-dependant animation
void idle(void * data, SoSensor * sensor){
	t+=FPSInterval;
	counter++;
	if(RANDOM&&!(counter%RANDOMINTERVAL)){
		int x=Random();x+=32768;x%=MAX_X;
		int y=Random();y+=32768;y%=MAX_Y;
		int z=Random();z+=32768;z%=MAX_Z;
		cubes[x][y][z+1].h=.1;
	}
			
	if(FORCE_FEEDBACK){
		//Force Feedback
		for(int z=MAX_Z-1;z>0;z--){
			//duplicate heightbuffer and Calculate Compression-Factor
			for(int x=0;x<MAX_X;x++){
				for(int y=0;y<MAX_Y;y++){
					heightbuffer2[x][y][z]=cubes[x][y][z].h;
					cubes[x][y][z].c=cubes[x][y][z+1].h+cubes[x][y][z+1].c;
				}
			}
			for(int x=0;x<MAX_X;x++){
				int lx=x-1;
				if(lx<0){lx=1;}
				int rx=x+1;
				if(rx>=MAX_X){rx=MAX_X-2;}
				for(int y=0;y<MAX_Y;y++){
					int dy=y-1;
					if(dy<0){dy=1;}
					int uy=y+1;
					if(uy>=MAX_Y){uy=MAX_Y-2;}
					//Updating the compression on the same level
					float compressed=cubes[x][y][z].h*COMPRESSION*(exp(-cubes[lx][y][z].c)+1);
					float putaway=compressed/4;
					heightbuffer2[lx][y][z]+=putaway;
					heightbuffer2[rx][y][z]+=putaway;
					heightbuffer2[x][dy][z]+=putaway;
					heightbuffer2[x][uy][z]+=putaway;
					heightbuffer2[x][y][z]-=compressed;
				}
			}
			//Copy back heightbuffer
			for(int x=0;x<MAX_X;x++){
				for(int y=0;y<MAX_Y;y++){
					cubes[x][y][z].h=heightbuffer2[x][y][z];
				}
			}
		}
	
	}	//end forcefeedback

	//Wave each level
	for(int z=MAX_Z;z>0;z--){
		for(int x=0;x<MAX_X;x++){
			int lx=x-1;
			if(lx<0){lx=1;}
			int rx=x+1;
			if(rx>=MAX_X){rx=MAX_X-2;}
			for(int y=0;y<MAX_Y;y++){
				int dy=y-1;
				if(dy<0){dy=1;}
				int uy=y+1;
				if(uy>=MAX_Y){uy=MAX_Y-2;}
				//Updating the depth on the same level
				float height=0;
				height+=-4*(cubes[x][y][z].h);
				height+=(cubes[lx][y][z].h);
				height+=(cubes[rx][y][z].h);
				height+=(cubes[x][uy][z].h);
				height+=(cubes[x][dy][z].h);
				heightbuffer2[x][y][z]=heightbuffer[x][y][z]+FPSInterval*WAVESPEED_2*height;
			}
		}
	}

	//Update Height and calculate Bottom
	for(int z=1;z<=MAX_Z;z++){
		//Calculate normalizing factor
		float n=0;
		for(int x=0;x<MAX_X;x++){
			for(int y=0;y<MAX_Y;y++){
				n+=cubes[x][y][z].h;
			}
		}
		n=INITIAL_HEIGHT*MAX_X*MAX_Y/n;
		//Update
		for(int x=0;x<MAX_X;x++){
			for(int y=0;y<MAX_Y;y++){
				cubes[x][y][z].h=(n*cubes[x][y][z].h)+FPSInterval*heightbuffer2[x][y][z];
				cubes[x][y][z].b=cubes[x][y][z-1].b+cubes[x][y][z-1].h;
			}
		}
	}
	
	//Copy back heightbuffer
	for(int z=0;z<=MAX_Z;z++){
		for(int x=0;x<MAX_X;x++){
			for(int y=0;y<MAX_Y;y++){
				heightbuffer[x][y][z]=heightbuffer2[x][y][z];
			}
		}
	}
	
	updatescene();
}

void updatescene(){
	for(int z=1;z<=MAX_Z;z++){
		for(int x=0;x<MAX_X;x++){
			for(int y=0;y<MAX_Y;y++){
				cubes[x][y][z].tran->translation.setValue(x,y,cubes[x][y][z].b+(cubes[x][y][z].h/2));
				cubes[x][y][z].cube->depth.setValue(BOXES_THICKNESS*(cubes[x][y][z].h));
			}
		}
	}
}