//Project.cpp
//Created by Lukas Fuellemann and Tobais Stamm
//The actual implementation of the project

#include "Project.h"
#include "Drum.h"
#include "Sphere.h"
#include "ListArray.h"
#include "ListElement.h"


float WaterMaterialAmbientColor[]={0,.5,1};
float WaterMaterialDiffuseColor[]={0,1,1};
float WaterMaterialSpecularColor[]={1,1,1};
float WaterMaterialEmissiveColor[]={0,.6,1};
float WaterMaterialShininess=1;
float WaterMaterialTransparency=.5;

//Constructor
Project::Project(SoSeparator * superroot){
	this->superroot=superroot;
	root=NULL;
	timer=NULL;
	drum=NULL;
	ss=NULL;
	nofspheres=START_NUM_SPHERES;
	firstparticle=NULL;
	show_particles=false;
	show_water=false;
	
	show_cubes=0;
	use_spatial_optimisation=1;

	//rebuild the scenegraph
	BuildSceneGraph();
}

//Destructor
Project::~Project(){
	Destruct();
}

void Project::Destruct(){
	//delete every object in the scenegraph
	superroot->removeAllChildren();
	if(root){root->unref();root=NULL;}
	
	if(timer){delete timer;timer=NULL;}
	if(drum){delete drum;drum=NULL;}
	if(ss){delete ss;ss=NULL;}
	t=0;
	counter=0;
	running=0;
	drum_rotation=0;
	drum_rotation_direction=0;
	watersep=NULL;
	watermat=NULL;
	watercoords=NULL;
	watermesh=NULL;
	if(firstparticle){delete firstparticle;firstparticle=NULL;}
	
	
	for(int y=0;y<MAX_WATER_Y;y++){
		for(int x=0;x<MAX_WATER_X;x++){
			water[MAX_WATER_X*y+x][0]=WATER_SIZE*(float)((float)x/(float)(MAX_WATER_X-1))-(WATER_SIZE/2);
			water[MAX_WATER_X*y+x][1]=WATER_HEIGHT;
			water[MAX_WATER_X*y+x][2]=WATER_SIZE*(float)((float)y/(float)(MAX_WATER_Y-1))-(WATER_SIZE/2);
			water2[MAX_WATER_X*y+x]=water[MAX_WATER_X*y+x][1];
		}
	}
}

//Starts/Stopps the timer
void Project::StartStopTimer(){
	if(timer){
		if(running){
			timer->unschedule();
			running=0;
		}else{
			timer->schedule();
			running=1;
		}
	}
}

/////////////////////
//Setting up the scene
void Project::BuildSceneGraph(){
	//First, delete everything and set everything to 0
	Destruct();
	
	//create a new root
	root = new SoSeparator();
	root->ref();

	//Create a new timer
	timer = new SoTimerSensor(::Idle,this);
	timer->setInterval(SbTime(FPSInterval));

	//Add helping functions for transparency
	hints = new SoShapeHints();
	hints->vertexOrdering = SoShapeHints::COUNTERCLOCKWISE;
	hints->shapeType = SoShapeHints::SOLID;
	root->addChild(hints);

	//Trommel definieren;
	drum = new Drum(DRUM_RADIUS);
	root->addChild(drum->getSep());

	//define the water
	watersep=new SoSeparator();
	watermat=new SoMaterial();
	watermat->ambientColor=SbVec3f(WaterMaterialAmbientColor);
	watermat->diffuseColor=SbVec3f(WaterMaterialDiffuseColor);
	watermat->specularColor=SbVec3f(WaterMaterialSpecularColor);
	watermat->emissiveColor=SbVec3f(WaterMaterialEmissiveColor);
	watermat->shininess=WaterMaterialShininess;
	if(show_water){
		watermat->transparency=WaterMaterialTransparency;
	}else{
		watermat->transparency=1;
	}
	watersep->addChild(watermat);
	
	watercoords=new SoCoordinate3;
	watercoords->point.setValues(0,MAX_WATER_X*MAX_WATER_Y,water);
	watersep->addChild(watercoords);
	watermesh=new SoQuadMesh();
	watermesh->verticesPerRow = MAX_WATER_X;
	watermesh->verticesPerColumn = MAX_WATER_Y;
	watersep->addChild(watermesh);
	root->addChild(watersep);

	//Redefine the spacial structure
	ss = new ListArray();

	//Kugeln definieren
	for(int i=0;i<nofspheres;i++){
		spheres[i]=createSphere(i);
	}
	
	//add everything to the superroot
	superroot->addChild(root);
}

Sphere* Project::createSphere(int newindex){
	Sphere* s=new Sphere(show_cubes);
	
	//Searching for a position without collision
	bool ok=false;
	while(!ok){
		ok=true;
		
		s->setRandomRadius();
		s->setRandomPos();
		for(int j=0;j<newindex;j++){
			SbVec3f dist=s->getPos() - spheres[j]->getPos();
			if(dist.length()<=(s->getRadius()+spheres[j]->getRadius())){ok=false;}
		}
	}
	
	s->setRandomVelocity(INIT_VELOCITY);
	s->setGravityAcceleration(-GRAVITY);
	s->setDefaultMass();

	s->setRandomColor();

	root->addChild(s->getSeparator());
	
	return s;
}

Sphere* Project::createParticle(SbVec3f& pos, SbVec3f& v){
	Sphere* s=new Sphere(true);
	
	float r=PARTICLE_SIZE*(rnd()*(1-RANDOMIZE_FACTOR));
	s->setRadius(r);
	s->setPos(pos[0]+rnd()*r,pos[1]+rnd()*r,pos[2]+rnd()*r);
	s->setV(v[0]+rnd()*.1,fabs((v[1]+rnd()*.1)/MAX_PARTICLES),v[2]+rnd()*.1);
	s->setGravityAcceleration(-GRAVITY);

	s->getMat()->ambientColor=SbVec3f(WaterMaterialAmbientColor);
	s->getMat()->diffuseColor=SbVec3f(WaterMaterialDiffuseColor);
	s->getMat()->specularColor=SbVec3f(WaterMaterialSpecularColor);
	s->getMat()->emissiveColor=SbVec3f(WaterMaterialEmissiveColor);
	s->getMat()->shininess=WaterMaterialShininess;
	s->getMat()->transparency=WaterMaterialTransparency;

	root->addChild(s->getSeparator());
	
	return s;
}

//returns a random value between 0 and 1
float Project::rnd(){
	return (((float)rand()/(float)RAND_MAX)*2)-1;
}

//Key-Handling
void Project::Key(short key){
	//printf("%i\n",(int)key);
	switch(key){
	case 'r':
		BuildSceneGraph();
		break;
	case 's':
		StartStopTimer();
		break;
	case 'o':
		if(use_spatial_optimisation){use_spatial_optimisation=0;}else{use_spatial_optimisation=1;}
		break;
	case '.':
		Idle();
		break;
	case 'b':
		drum_rotation_direction-=.002;
		break;
	case 'v':
		drum_rotation_direction+=.002;
		break;
	case 'B':
		drum_rotation_direction-=.02;
		break;
	case 'V':
		drum_rotation_direction+=.02;
		break;
	case 'p':
		if(show_particles){show_particles=false;}else{show_particles=true;};
		break;
	case 'w':
		if(show_water){
			show_water=false;
			watermat->transparency=1;
		}else{
			show_water=true;
			watermat->transparency=WaterMaterialTransparency;
		}
		break;
	case 'y':
		if(nofspheres>0){
			spheres[nofspheres-1]->unset(root);
			delete spheres[nofspheres-1];
			nofspheres--;
		}
		break;
	case 'x':
		if(nofspheres<(MAX_SPHERES-1)){
			spheres[nofspheres]=createSphere(nofspheres);
			nofspheres++;
		}
		break;
	case 'Y':
		for(int i=0;i<10;i++){
			if(nofspheres>0){
				spheres[nofspheres-1]->unset(root);
				delete spheres[nofspheres-1];
				nofspheres--;
			}
		}
		break;
	case 'X':
		for(int i=0;i<10;i++){
			if(nofspheres<(MAX_SPHERES-1)){
				spheres[nofspheres]=createSphere(nofspheres);
				nofspheres++;
			}
		}
		break;
	case 'c':
		if(show_cubes){show_cubes=0;}else{show_cubes=1;}
		for(int i=0;i<nofspheres;i++){
			spheres[i]->switchSphereCube(show_cubes);
		}
		break;
	default:
		break;
	}
}

//The idle-function. Calculating the t-dependant animation
void Project::Idle()
{

	ss->CleanAll();
	for(int i=0;i<nofspheres;i++){
		ss->Add(spheres[i]);
	}

	t+=FPSInterval;
	counter++;
	drum_rotation+=drum_rotation_direction;
	if(drum_rotation>(2*PI)){drum_rotation-=2*PI;}
	if(drum_rotation<0){drum_rotation+=2*PI;}
	
	//update the drum
	drum->getTran()->rotation.setValue(SbVec3f(0,0,1),drum_rotation);

	if(use_spatial_optimisation){
		for(int i=0;i<nofspheres-1;i++){
			ss->Traverse(spheres[i],Collide);
		}
	}else{
	//	old function for checking every sphere against every other
		for(int i=0;i<nofspheres;i++){
			for(int j=0;j<i;j++){
				Sphere * s1=spheres[i];
				Sphere * s2=spheres[j];
				SbVec3f dist=s1->getPos() - s2->getPos();
				if(dist.length()<=(s1->getRadius()+s2->getRadius())){
					SbVec3f s1v=s1->getV();
					SbVec3f s2v=s2->getV();
					float m1=s1->getMass();
					float m2=s2->getMass();

					dist.normalize();
					SbVec3f v1x=dist*dist.dot(s1v);
					SbVec3f v1y=s1v-v1x;
					SbVec3f v2x=dist*dist.dot(s2v);
					SbVec3f v2y=s2v-v2x;
					SbVec3f v1xnew=(m1-m2)/(m1+m2)*v1x+(2*m2)/(m1+m2)*v2x;
					SbVec3f v2xnew=(2*m1)/(m1+m2)*v1x+(m2-m1)/(m1+m2)*v2x;
					s1->setV(v1xnew+v1y);
					s2->setV(v2xnew+v2y);
				}
			}
		}
	}

	//euler
	ListElement* curparticle=firstparticle;
	while(curparticle){
		if(curparticle->getSphere()){
			curparticle->getSphere()->Euler(FPSInterval);
		}
		curparticle=curparticle->getNext();
	}
	for(int i=0;i<nofspheres;i++){
		if(spheres[i]->getPos()[1]<WATER_HEIGHT){
			spheres[i]->Euler(FPSInterval);
		}else{
			spheres[i]->Euler(FPSInterval);
		}
	}

	//collistion detection on the wall
	float interval=FPSInterval;
	for(int i=0;i<nofspheres;i++){
		Sphere* s=spheres[i];
		SbVec3f opos=s->getPos();
		SbVec3f ov=s->getV();
		SbVec3f oa=s->getA();
		SbVec3f pos=opos;
		SbVec3f v=ov;
		SbVec3f a=oa;
		rotate(pos,-drum_rotation);
		rotate(v,-drum_rotation);
		rotate(a,-drum_rotation);
		SbVec3f f;
		
		float r=s->getRadius();
		if((pos[0]+r)>=DRUM_RADIUS){
			//printf("Kollision rechts");
			v[0]=-v[0]*(1-WALL_DAMPING);
			f=a*interval;
			f[0]*=-(pos[1]/DRUM_RADIUS)*WALL_BUMPING*drum_rotation_direction;
			v += f;
			//v[0]*=(1-WALL_DAMPING);
			pos[0]-=(1-WALL_DAMPING)*(pos[0]+r-DRUM_RADIUS);
		}
		if((pos[0]-r)<=-DRUM_RADIUS){
			//printf("Kollision links");
			v[0]=-v[0]*(1-WALL_DAMPING);
			f=a*interval;
			f[0]*=(pos[1]/DRUM_RADIUS)*WALL_BUMPING*drum_rotation_direction;
			v += f;
			//v[0]*=(1-WALL_DAMPING);
			pos[0]+=(1-WALL_DAMPING)*(-pos[0]+r-DRUM_RADIUS);
		}
		if((pos[1]+r)>=DRUM_RADIUS){
			//printf("Kollision oben");
			v[1]=-v[1]*(1-WALL_DAMPING);
			f=a*interval;
			f[1]*=(pos[0]/DRUM_RADIUS)*WALL_BUMPING*drum_rotation_direction;
			v += f;
			//v[1]*=(1-WALL_DAMPING);
			pos[1]-=(1-WALL_DAMPING)*(pos[1]+r-DRUM_RADIUS);
		}
		if((pos[1]-r)<=-DRUM_RADIUS){
			//printf("Kollision unten");
			v[1]=-v[1]*(1-WALL_DAMPING);
			f=a*interval;
			f[1]*=-(pos[0]/DRUM_RADIUS)*WALL_BUMPING*drum_rotation_direction;
			v += f;
			//v[1]*=(1-WALL_DAMPING);
			pos[1]+=(1-WALL_DAMPING)*(-pos[1]+r-DRUM_RADIUS);
		}
		if((pos[2]+r)>=DRUM_RADIUS){
			//printf("Kollision hinten");
			v[2]=-v[2]*(1-WALL_DAMPING);
			v += a * interval;
			//v[2]*=(1-WALL_DAMPING);
			pos[2]-=(1-WALL_DAMPING)*(pos[2]+r-DRUM_RADIUS);
		}
		if((pos[2]-r)<=-DRUM_RADIUS){
			//printf("Kollision vorne");
			v[2]=-v[2]*(1-WALL_DAMPING);
			v += a * interval;
			//v[2]*=(1-WALL_DAMPING);
			pos[2]+=(1-WALL_DAMPING)*(-pos[2]+r-DRUM_RADIUS);
		}
		rotate(v,drum_rotation);
		rotate(pos,drum_rotation);
		s->setV(v);
		s->setPos(pos);
	}

	//collision-detection with the water
	
	for(int i=0;i<nofspheres;i++){
		SbVec3f pos=spheres[i]->getPos();
		float r=spheres[i]->getRadius();
		if(pos[1]<(WATER_HEIGHT+r)){
			SbVec3f v=spheres[i]->getV();
			v*=(1-WATER_SPHERE_DAMPING);
			spheres[i]->setV(v);
			float xpos=((pos[0]/(float)(WATER_SIZE/2))+1)/2;
			float zpos=((pos[2]/(float)(WATER_SIZE/2))+1)/2;
			float influence;
			if(pos[1]<-DRUM_RADIUS){
				influence=0;
			}else{
				influence=((pos[1]+DRUM_RADIUS)/WATER_LEVEL);
			}
				water2[(int)(zpos*MAX_WATER_Y)*MAX_WATER_X+(int)(xpos*MAX_WATER_X)]+=(spheres[i]->getV()[1])*r*SPHERE_DROP*influence*WATER_WAVESIZE;
			
			if(pos[1]>(WATER_HEIGHT-r)){
				//create particles
				if(show_particles){
					for(int p=0;p<(int)(r/MAX_RADIUS*MAX_PARTICLES);p++){
						SbVec3f v=spheres[i]->getV();
						Sphere* part=createParticle(pos,v);
						ListElement* elem=new ListElement(part);
						elem->setNext(firstparticle);
						firstparticle=elem;
					}
				}
			}
		}
	}

	//Waving the water
	for(int x=0;x<MAX_WATER_X;x++){
		for(int y=0;y<MAX_WATER_Y;y++){
			int lx=x-1;
			if(lx<0){lx=1;}
			int rx=x+1;
			if(rx>=MAX_WATER_X){rx=MAX_WATER_X-2;}
			int ly=y-1;
			if(ly<0){ly=1;}
			int ry=y+1;
			if(ry>=MAX_WATER_Y){ry=MAX_WATER_Y-2;}
			float height=0;
			height+=water[y*MAX_WATER_X+lx][1];
			height+=water[y*MAX_WATER_X+rx][1];
			height+=water[ly*MAX_WATER_X+x][1];
			height+=water[ry*MAX_WATER_X+x][1];
			height-=(4*water[y*MAX_WATER_X+x][1]);
			water3[y*MAX_WATER_X+x]=water2[y*MAX_WATER_X+x]+FPSInterval*WAVE_SPEED*height;
		}
	}
	//normalizing:
	float n=0;
	for(int y=0;y<MAX_WATER_Y;y++){
		for(int x=0;x<MAX_WATER_X;x++){
			n+=water[y*MAX_WATER_X+x][1];
		}
	}
	n=WATER_HEIGHT*MAX_WATER_X*MAX_WATER_Y/n;
	//updating
	for(int x=0;x<MAX_WATER_X;x++){
		for(int y=0;y<MAX_WATER_Y;y++){
			water[y*MAX_WATER_X+x][1]=n*water[y*MAX_WATER_X+x][1]+FPSInterval*water3[y*MAX_WATER_X+x];
			water2[y*MAX_WATER_X+x]=(water3[y*MAX_WATER_X+x]);
		}
	}
	watercoords->point.setValues(0,MAX_WATER_X*MAX_WATER_Y,water);

	//Particle-Update.
	curparticle=firstparticle;
	while(curparticle){
		if(curparticle->getSphere()){
			Sphere* s=curparticle->getSphere();
			if(s->getPos()[1]<-DRUM_RADIUS){
				s->unset(root);curparticle->nullSphere();
			}else{
				s->getTran()->rotation.setValue(rnd(),rnd(),rnd(),rnd());
			}
		}
		curparticle=curparticle->getNext();
	}

		//delete all Elements which have no more sphere defined
	while((firstparticle)&&(!firstparticle->getSphere())){
		ListElement * nextparticle=firstparticle->getNext();
		firstparticle->setNext(NULL);
		delete firstparticle;
		firstparticle=nextparticle;
	}

}

void Project::rotate(SbVec3f& x, float rot){
	SbVec3f x2;
	x2[0]=cos(rot)*x[0]-sin(rot)*x[1];
	x2[1]=sin(rot)*x[0]+cos(rot)*x[1];
	x2[2]=x[2];
	x[0]=x2[0];
	x[1]=x2[1];
	x[2]=x2[2];
}

//////////////////
//The idle-function. Calculating the t-dependant animation
void Idle(void * data, SoSensor * sensor){
	Project * project = (Project*)data;
	project->Idle();
}

//The keyboard-Function.
void KeyboardEvent(void * data, SoEventCallback * cb){
	Project * project = (Project*)data;
	project->Key(((SoKeyboardEvent*)cb)->getKey());
}

void Collide(Sphere * s1, Sphere * s2){

	float dt = (float)FPSInterval;
	SbVec3f t1 = s1->getPos() + s1->getV() * dt;
	SbVec3f t2 = s2->getPos() + s2->getV() * dt;
	SbVec3f dist = t2 - t1;

	//ab hier nichts mehr geändert; Pos der Sphere wird nicht verändert
	if(dist.length()<=(s1->getRadius() + s2->getRadius())){
		SbVec3f s1v=s1->getV();
		SbVec3f s2v=s2->getV();
		float m1=s1->getMass();
		float m2=s2->getMass();

		dist.normalize();
		SbVec3f v1x=dist*dist.dot(s1v);
		SbVec3f v1y=s1v-v1x;
		SbVec3f v2x=dist*dist.dot(s2v);
		SbVec3f v2y=s2v-v2x;
		SbVec3f v1xnew=(m1-m2)/(m1+m2)*v1x+(2*m2)/(m1+m2)*v2x;
		SbVec3f v2xnew=(2*m1)/(m1+m2)*v1x+(m2-m1)/(m1+m2)*v2x;
		s1->setV(v1xnew+v1y);
		s2->setV(v2xnew+v2y);
	}
}

