//Project.h
//Created by Lukas Fuellemann and Tobais Stamm
//The include-file of the project

/////////////////////
//INCLUDES

#include <Inventor/nodes/SoNodes.h>
#include <Inventor/sensors/SoTimerSensor.h>
#include <Inventor/events/SoKeyboardEvent.h>
#include <Inventor/events/SoMouseButtonEvent.h>

#include "Globals.h"

class ListArray;
class ListElement;
class Sphere;
class Drum;
class SCSceneGraph;

class Project{

SoSeparator* superroot;					//a pointer to root-node of the scenegraph
SoSeparator* root;							//the root node
SoTimerSensor * timer;					//the timer for the idle-callback
SoShapeHints * hints;						//helping hints for transparency
Sphere* spheres[MAX_SPHERES];		//the array storing all spheres
Drum * drum;										//the drum
ListArray* ss;									//the spacial-structure
float t;												//universal time
int counter;										//universal counter
char running;										//identifier, if scene is running
float drum_rotation;						//the actual rotation of the drum
float drum_rotation_direction;	//the timestep of the rotation
float water[MAX_WATER_X*MAX_WATER_Y][3];	//The Water
float water2[MAX_WATER_X*MAX_WATER_Y];			//The Water-updating-heights
float water3[MAX_WATER_X*MAX_WATER_Y];			//The Water-updating-heights2
SoSeparator* watersep;							//the separator for the water
SoMaterial* watermat;								//the material of the water
SoCoordinate3* watercoords;					//the coordinates of the water
SoQuadMesh* watermesh;							//the mesh of the water
int nofspheres;											//actual number of spheres
ListElement* firstparticle;					//the first particle

char show_cubes;								//cubes or spheres
char use_spatial_optimisation;	//switches between optimisation
char show_particles;						//switch for particles
char show_water;						//switch for particles

public:

//Constructor
Project(SoSeparator * superroot);
//Destructor
~Project();

//cleans up all the mess and inits to 0
void Destruct();
//Starts or stops the timer
void StartStopTimer();
//Creates the Starting-Scene
void BuildSceneGraph();
//The idle-Function
void Idle();
//The Keyboard-Function
void Key(short key);
//Rotate a vector
void rotate(SbVec3f& x, float rot);
//Creates a Sphere
Sphere* createSphere(int newindex);
//creates a particle
Sphere* createParticle(SbVec3f& pos,SbVec3f& v);

//Gives back a random number between 0 and 1
static float rnd();

};

//The idle-Function. Is called while drawing every new frame
void Idle(void * data, SoSensor * sensor);
//The keyboard-Callback-Event.
void KeyboardEvent(void * data, SoEventCallback * cb);
//collision-detection and response
void Collide(Sphere* sphere1, Sphere* sphere2);
