//Sphere.cpp
//Created by Lukas Fuellemann and Tobais Stamm
//The file of the Sphere-implementation

#include "Globals.h"
#include "Sphere.h"
#include "Project.h"
#include <Inventor/nodes/SoNodes.h>


//Constructor
Sphere::Sphere(bool show_cubes){
	//create all new objects
	sep=new SoSeparator();
	tran=new SoTransform();
	mat=new SoMaterial();
	m=0;
	r=0;
	setSpecularColor(1,1,1);
	setShininess(.4);
	setTransparency(0);
	this->show_cubes=show_cubes;


	//put them together
	sep->addChild(tran);
	sep->addChild(mat);

	if(show_cubes){
		cube=new SoCube();
		sep->addChild(cube);
	}else{
		sphere=new SoSphere();
		sep->addChild(sphere);
	}
}

Sphere::~Sphere(){
	//nothing to do. Elements will be deleted by root->unref
}

void Sphere::switchSphereCube(char show_cubes){
	if(show_cubes){
		sep->removeChild(sphere);
		cube=new SoCube();
		sep->addChild(cube);
	}else{
		sep->removeChild(cube);
		sphere=new SoSphere();
		sep->addChild(sphere);
	}
	this->show_cubes=show_cubes;
	setRadius(this->r);
}

//removes the sphere/cube from the root
void Sphere::unset(SoSeparator* root){
	root->removeChild(sep);
}

//Set the radius of the sphere
void Sphere::setRadius(float r){
	this->r=r;
	if(show_cubes){
		cube->width=r;
		cube->height=r;
		cube->depth=r;
	}else{
		sphere->radius.setValue(r);
	}
}

//Set the position of the sphere
void Sphere::setPos(float x, float y, float z){
	pos.setValue(x,y,z);
	tran->translation.setValue(x,y,z);
}

//Set the position of the sphere
void Sphere::setPos(SbVec3f pos){
	this->pos.setValue(pos);
	tran->translation.setValue(pos);
}

//Sets the position randomly
void Sphere::setRandomPos(){
	setPos(Project::rnd()*(DRUM_RADIUS-r),Project::rnd()*(DRUM_RADIUS-r),Project::rnd()*(DRUM_RADIUS-r));
}

//Sets the radius randomly
void Sphere::setRandomRadius(){
	setRadius(MAX_RADIUS*((1-RANDOMIZE_FACTOR)+RANDOMIZE_FACTOR*(Project::rnd()+1)/2));
}

//Sets the velocity randomly
void Sphere::setRandomVelocity(float t){
	v.setValue(Project::rnd()*t,Project::rnd()*t,Project::rnd()*t);
}

//sets the acceleration to the normal gravity
void Sphere::setGravityAcceleration(float g){
	a.setValue(0,g,0);
}

//sets the mass of the sphere to the default mass defined by the radius
void Sphere::setDefaultMass(){
	m=r*r*r*PI*4/3;
}

//Set the color of the sphere to a random value
void Sphere::setRandomColor(){
		mat->diffuseColor.setValue((Project::rnd()+1)/2,(Project::rnd()+1)/2,(Project::rnd()+1)/2);
}

//performs one Euler-Step
void Sphere::Euler(float dt){
	pos += v * dt;
	tran->translation.setValue(pos);
	v += a * dt;
}