//Sphere.h
//Created by Lukas Fuellemann and Tobais Stamm
//The include-file of the Sphere

class SoSeparator;
class SoTransform;
class SoMaterial;
class SoSphere;
class SbVec3f;
#include <Inventor/SbLinear.h>
#include <Inventor/nodes/SoNodes.h>

class Sphere{

SoSeparator* sep;				//The separator of one sphere
SoTransform* tran;			//the translation of the sphere
SoMaterial* mat;				//the material of the sphere
SoSphere* sphere;				//the sphere itself
SoCube* cube;						//a cube
SbVec3f pos;						//the position of the sphere. 3-Vector
SbVec3f v;							//the current velocity of the sphere.
SbVec3f a;							//the current acceleration of the sphere.
float m;								//the mass of the sphere
float r;								//the radius of the sphere
bool show_cubes;				//are cubes shown?

public:

Sphere(bool show_cubes);
~Sphere();


void switchSphereCube(char show_cubes);

//Set the radius of the sphere
void setRadius(float r);
//Set the position of the sphere
void setPos(float x, float y, float z);
void setPos(SbVec3f pos);

SbVec3f Sphere::getPos(){return pos;}
float Sphere::getRadius(){return r;}
SbVec3f Sphere::getV(){return v;}
SbVec3f Sphere::getA(){return a;}
float Sphere::getMass(){return m;}
SoTransform* Sphere::getTran(){return tran;}
SoSeparator* Sphere::getSeparator(){return sep;}
SoSphere* Sphere::getSphere(){return sphere;}
SoCube* Sphere::getCube(){return cube;}
SoSeparator* Sphere::getSep(){return sep;}
SoMaterial* Sphere::getMat(){return mat;}

//Sets the position randomly
void setRandomPos();
//Sets the radius randomly
void setRandomRadius();
//sets the velocity
void Sphere::setV(SbVec3f v){this->v.setValue(v);}
void Sphere::setV(float x, float y, float z){this->v.setValue(x,y,z);}
//Sets the velocity randomly
void setRandomVelocity(float t);
//sets the acceleration to the normal gravity
void setGravityAcceleration(float g);
//sets the mass of the sphere to the default mass defined by the radius
void setDefaultMass();
//Set the color of the sphere to a random value
void setRandomColor();
//set the specular color
void Sphere::setSpecularColor(float r, float g, float b){mat->specularColor.setValue(r, g, b);}
//sets the shininess of the sphere
void Sphere::setShininess(float s){mat->shininess.setValue(s);}
//sets the transperency of the sphere
void Sphere::setTransparency(float a){mat->transparency.setValue(a);}
//removes the sphere/cube from the root
void unset(SoSeparator* root);

//performs one Euler-Step
void Euler(float dt);

//Gives back the minimum/maximum x/y/z-coordinate
float Sphere::Maxx(){return pos[0]+r;}
float Sphere::Maxy(){return pos[1]+r;}
float Sphere::Maxz(){return pos[2]+r;}
float Sphere::Minx(){return pos[0]-r;}
float Sphere::Miny(){return pos[1]-r;}
float Sphere::Minz(){return pos[2]-r;}

};